package cloud.javastudy.demo.aop;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;

/**
 * 声明一个切面，只用加一个@Aspect注解就可以了
 * 还有个注解@Component是声明这个类为Spring的bean
 */
@Aspect
@Component
public class TestAspect {

    /**
     * 切面的组成部分之一，告诉程序要对哪些方法进行操作
     * 在DEMO中写死了一个方法，下面会给出常用的切面写法
     * 在现在注解盛行的时代，这样根据方法名去找方法的用处已经不多了
     * 更多的是通过某个注解去找到对应的方法，后续也会有DEMOb出来
     */
    @Pointcut("execution(public * cloud.javastudy.demo.controller.TestController.testAspect(..))")
    public void testPointcut(){}

    /**
     * 对找到的方法进行修改
     * 有@Around，对方法执行前后进行修改
     * 有@Before, 在方法执行前进行修改
     * 有@After,  在方法执行后进行修改
     * 有@AfterReturning， 在方法返回后进行修改
     * 有@AfterThrowing， 在方法抛出异常后修改
     * 常用的是@Around,常用于对方法体加 Try-catch，对方法执行计时，对方法入参，返回做日志记录等等
     */
    @Around("testPointcut()")
    public Object doAround(ProceedingJoinPoint joinPoint) throws Throwable {
        System.out.println("before pointcut");
        Object proceed = joinPoint.proceed();
        System.out.println("after pointcut");
        return proceed;
    }
}
